public class Circle
{

	// Fields
	private double x; // center x coord
	private double y; // center y coord
	private double radius; // circle radius

	// toString
	public String toString()
	{
		// create a string to return
		String s = "Circle:x="+this.x+",y="+this.y+",radius="+this.radius;
		return s;
	}

	// Creates a circle with center at the 
	// given x-y position, with the given radius. 
	public Circle(double x, double y, double radius)
	{
		this.x = x;
		this.y = y;
		this.radius = radius;
	}

	// Creates a circle at the origin with radius 1. 
	public Circle()
	{
		this(0,0,1);
	}

	//Returns the X coordinate of the circle's center.
	public double getX()
	{
		// accessor
		return this.x;
	}

	public double getY()
	{
		return this.y;
	}

	public double getRadius()
	{
		return this.radius;
	}

	//Determines whether this circle 
	//overlaps with another given circle.
	public boolean overlap(Circle otherCircle)
	{
		// get the sum of radii
		// radius of "this" circle + 
		// radius of otherCircle
		double sr = this.radius + otherCircle.radius;

		// get the square of the distance between the 2 circles
		double d = (this.y-otherCircle.y)*(this.y-otherCircle.y);
		d = d + (this.x-otherCircle.x)*(this.x-otherCircle.x);

		if (d < (sr*sr))
		{
			// the circles overlap
			return true;
		}
		else
		{
			// they do not overlap
			return false;
		}

	}

	///////////////

	// Contains the value of pi to four decimal places. 
	public static final double PI = 3.1416;

	// Calculates the circumference of a circle with the given radius. 
	public static double circum(double radius)
	{
		//double c; // contains the circumference
		//c = 2 * Circle.PI * radius; // 2 * pi * r
		//return c;
		return 2*Circle.PI*radius;
	}

	// Calculates (base^exp), where exp is a nonnegative integer. 
	private static double intPow(double base, int exp)
	{
		double result = 1;
		for (int i = 0; i < exp; i++)
		{
			result = result * base;
		}
		return result;
	}

	// Calculates the area of a circle with the given radius. 
	public static double area(double radius)
	{
		// pi * r^2
		return Circle.PI * Circle.intPow(radius,2);
	}
}
