public class MutablePoint
{
	private double x;
	private double y;

	/**
		Creates a point with the given x-y coordinates.
		@param x x coordinate
		@param y y coordinate
	*/
	public MutablePoint(double x, double y)
	{
		this.x = x;
		this.y = y;
	}

	/**
		Creates a point at the origin (x=y=0).
	*/
	public MutablePoint(){ this(0,0); }

	/**
		Copy constructor.
		@param p MutablePoint object to be copied.
	*/
	public MutablePoint(MutablePoint p){ this(p.x,p.y); }

	/**
		Returns the value of x.
		@return x coordinate
	*/
	public double getX(){ return this.x; }

	/**
		Returns the value of y.
		@return y coordinate
	*/
	public double getY(){ return this.y; }

	/**
		Sets the value of the coordinates.
		@param x New X coordinate.
		@param y New Y coordinate.
	*/
	public void set(double x,double y)
	{ 
		this.x = x; 
		this.y = y;
	}

	/**
		Converts the point to a string.  For example, if x=y=0,
		returns "[MutablePoint:x=0.0,y=0.0]".
		@return String version of the point.
	*/
	public String toString()
	{
		return "[MutablePoint:x="+this.x+",y="+this.y+"]";
	}

	/**
		Determines whether two points are equal.
		The points are equal if they are at the same coordinates.
		@return True if equal, false otherwise.
	*/	
	public boolean equals(Object o)
	{
		if (o instanceof MutablePoint)
		{
			MutablePoint p = (MutablePoint)o;
			if ((p.x == this.x)&&(p.y == this.y))
			{
				return true;
			}
		}
		return false;
	}

}
