public class LinkedList<E>
{

	private Node<E> head;

	public LinkedList()
	{
		// empty list
		head = null;
	}

	// walk through the list
	// and get the node at a given position
	// if there is no node at that position
	// i.e. the position is outside the list
	// then return null
	private Node<E> walk(int position)
	{
		Node<E> result = this.head;
		for (int i = 0; i < position; i++)
		{
			// walk through the list
			// "position" number of times
			if (result == null) return null;
			result = result.getNext();
		}
		return result;
	}

	// return data at position
	// if position out of range, return null
	public E getData(int position)
	{
		Node<E> n = walk(position);
		if (n == null)
		{
			// position is out of range
			return null;
		}
		return n.getData();
	}

	// add newNode to position
	// position is the new position of newNode
	// returns true if add is successful, false otherwise
	public boolean add(int position, Node<E> newNode)
	{
		if (position > 0)
		{
			// get element at position-1
			Node<E> prev = this.walk(position-1);

			if (prev == null) return false; 

			// prev.next has to point to newNode
			// newNode.next has to point to 
			// the old prev.next
			newNode.setNext(prev.getNext());
			prev.setNext(newNode);
			return true;
		}
		else
		{
			newNode.setNext(this.head);
			this.head = newNode;
			return true;
		}
	}

	// Delete node at given position, if it exists
	// Returns true if successful, false otherwise
	public boolean delete(int position)
	{
		if (position > 0)
		{
			Node<E> delNode = walk(position);
			Node<E> prevNode = walk(position-1);

			if ((delNode == null)||(prevNode == null))
			{
				return false;
			}
			prevNode.setNext(delNode.getNext());
			return true;
		}
		else
		{
			if (head == null)
			{
				return false;
			}
			this.head = this.head.getNext();
			return true;
		}
	}
}
