public class LinkedList<E>
{

	private Node<E> head;

	/**
		Creates an empty linked list.
	*/
	public LinkedList()
	{
		// empty list
		head = null;
	}

	// walk through the list
	// and get the node at a given position
	// if there is no node at that position
	// i.e. the position is outside the list
	// then return null
	private Node<E> walk(int position)
	{
		Node<E> result = this.head;
		for (int i = 0; i < position; i++)
		{
			// walk through the list
			// "position" number of times
			if (result == null) return null;
			result = result.getNext();
		}
		return result;
	}

	/**
		Returns the data at the given position of the linked list.
		If the position is out of range, returns null.
		Remeber that position 0 is the first position in the list.
		@param position Position of data to be returned
		@return Data at the given position
	*/
	public E getData(int position)
	{
		Node<E> n = walk(position);
		if (n == null)
		{
			// position is out of range
			return null;
		}
		return n.getData();
	}

	/**
		Adds a node to the list; the new node is at the given position.
		Returns true if the add is successful, false otherwise.
		@param position Position of new node
		@param newNode New node to be added to the list
		@return True if add is successful, false otherwise.
	*/
	public boolean add(int position, Node<E> newNode)
	{
		if (position > 0)
		{
			// get element at position-1
			Node<E> prev = this.walk(position-1);

			if (prev == null) return false; 

			// prev.next has to point to newNode
			// newNode.next has to point to 
			// the old prev.next
			newNode.setNext(prev.getNext());
			prev.setNext(newNode);
			return true;
		}
		else
		{
			newNode.setNext(this.head);
			this.head = newNode;
			return true;
		}
	}

	/**
		Deletes the node at the given position.  Returns true if successful, false otherwise.
		@param position Position to delete
		@return True if successful, false otherwise.
	*/
	public boolean delete(int position)
	{
		if (position > 0)
		{
			Node<E> delNode = walk(position);
			Node<E> prevNode = walk(position-1);

			if ((delNode == null)||(prevNode == null))
			{
				return false;
			}
			prevNode.setNext(delNode.getNext());
			return true;
		}
		else
		{
			if (head == null)
			{
				return false;
			}
			this.head = this.head.getNext();
			return true;
		}
	}

	/**
		Returns the number of nodes in the list.
		@return Number of nodes in the list.
	*/
	public int size()
	{
		return 0;
	}

	/**
		Prints the elements of the list, one per line.
	*/
	public void printList()
	{}

	/**
		Searches the list for an element equal to the given element.
		Returns the index of the first found element, or -1 if the element is not found.
		@return Index of the first found element, or -1 if the element is not found.
	*/
	public int search(E target)
	{
		return 0;
	}

}
