public class LinkedList<E extends Comparable>
{
	private int size;
	private Node<E> head;

	public LinkedList()
	{
		// empty list
		this.head = null;
		this.size = 0;
	}

	// walk through the list
	// and get the node at a given position
	// if there is no node at that position
	// i.e. the position is outside the list
	// then return null
	private Node<E> walk(int position)
	{
		Node<E> result = this.head;
		for (int i = 0; i < position; i++)
		{
			// walk through the list
			// "position" number of times
			if (result == null) return null;
			result = result.getNext();
		}
		return result;
	}

	// return data at position
	// if position out of range, return null
	public E getData(int position)
	{
		Node<E> n = walk(position);
		if (n == null)
		{
			// position is out of range
			return null;
		}
		return n.getData();
	}

	// add newNode to position
	// position is the new position of newNode
	// returns true if add is successful, false otherwise
	public boolean add(int position, E newData)
	{
		Node<E> newNode = new Node<E>(newData,null);
		if (position > 0)
		{
			// get element at position-1
			Node<E> prev = this.walk(position-1);

			if (prev == null) return false; 

			// prev.next has to point to newNode
			// newNode.next has to point to 
			// the old prev.next
			newNode.setNext(prev.getNext());
			prev.setNext(newNode);
			this.size++;
			return true;
		}
		else
		{
			newNode.setNext(this.head);
			this.head = newNode;
			this.size++;
			return true;
		}
	}

	// Walks through the list until it finds
	// an element greater than newData,
	// then adds the data at that position
	public void addInOrder(E newData)
	{
		Node current = head;
		int counter = 0;
		while ((current != null)&&(newData.compareTo(current.getData()) >= 0))
		{
			current = current.getNext();
			counter++;
		}
		this.add(counter,newData);
	}
		

	// Delete node at given position, if it exists
	// Returns true if successful, false otherwise
	public boolean delete(int position)
	{
		if (position > 0)
		{
			Node<E> delNode = walk(position);
			Node<E> prevNode = walk(position-1);

			if ((delNode == null)||(prevNode == null))
			{
				return false;
			}
			prevNode.setNext(delNode.getNext());
			this.size--;
			return true;
		}
		else
		{
			if (head == null)
			{
				return false;
			}
			this.head = this.head.getNext();
			this.size--;
			return true;
		}
	}

	// return the size of the list
	public int size()
	{
		return this.size;
	}

	// Print all the elements of the list, one per line
	public void printList()
	{
		Node<E> current = head;
		while (current != null)
		{
			System.out.println(current.getData());
			current = current.getNext();
		}	
	}

	// Search the list for target.
	// If found, return the position of the first match.
	// If not found, return -1.
	public int search(E target)
	{
		Node<E> current = head;
		int steps = 0;
		while (current != null)
		{
			// check for equality between
			// target and the data at current
			E data = current.getData();
			if (data.equals(target)) return steps;
			steps++;
			current = current.getNext();
		}

		return -1;
	}
}
