public class LinkedList
{
	private class Node
	{
		private String data;
		private LinkedList.Node next;

		private Node(String data, LinkedList.Node next)
		{
			this.data = data;
			this.next = next;
		}
	}

	private int size;
	private LinkedList.Node head;

	public LinkedList()
	{
		// empty list
		this.head = null;
		this.size = 0;
	}

	// walk through the list
	// and get the node at a given position
	// if there is no node at that position
	// i.e. the position is outside the list
	// then return null
	private LinkedList.Node walk(int position)
	{
		Node result = this.head;
		for (int i = 0; i < position; i++)
		{
			// walk through the list
			// "position" number of times
			if (result == null) return null;
			result = result.next;
		}
		return result;
	}

	// return data at position
	// if position out of range, return null
	public String getData(int position)
	{
		LinkedList.Node n = walk(position);
		if (n == null)
		{
			// position is out of range
			return null;
		}
		return n.data;
	}

	// add newNode to position
	// position is the new position of newNode
	// returns true if add is successful, false otherwise
	public boolean add(int position, String newData)
	{
		LinkedList.Node newNode = new LinkedList.Node(newData,null);
		if (position > 0)
		{
			// get element at position-1
			LinkedList.Node prev = this.walk(position-1);

			if (prev == null) return false; 

			// prev.next has to point to newNode
			// newNode.next has to point to 
			// the old prev.next
			newNode.next = prev.next;
			prev.next = newNode;
			this.size++;
			return true;
		}
		else
		{
			newNode.next = this.head;
			this.head = newNode;
			this.size++;
			return true;
		}
	}

	// Delete node at given position, if it exists
	// Returns true if successful, false otherwise
	public boolean delete(int position)
	{
		if (position > 0)
		{
			LinkedList.Node delNode = walk(position);
			LinkedList.Node prevNode = walk(position-1);

			if ((delNode == null)||(prevNode == null))
			{
				return false;
			}
			prevNode.next = delNode.next;
			this.size--;
			return true;
		}
		else
		{
			if (head == null)
			{
				return false;
			}
			this.head = this.head.next;
			this.size--;
			return true;
		}
	}

	// return the size of the list
	public int size()
	{
		return this.size;
	}

	// Print all the elements of the list, one per line
	public void printList()
	{
		LinkedList.Node current = head;
		while (current != null)
		{
			System.out.println(current.data);
			current = current.next;
		}	
	}

}
